#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include "pcc_int.h"

/***********************************************************************************/
/*       button services                                          				   */
/***********************************************************************************/

#define PCC_REPEAT_MASK     (1<<0 | 1<<1)       // repeat: key0, key1
#define PCC_REPEAT_START    50                        // after 500ms
#define PCC_REPEAT_NEXT     20                        // every 200ms

/***********************************************************************************/
/* the function initializes the vars for the button services					   */
/***********************************************************************************/
void	pcc_key_init	(void)
{
	//pcc_data.keys.rpt   = 0
	pcc_data.keys.state	= 0;
	pcc_data.keys.press	= 0;
}
 
/***********************************************************************************/
/* each key pressed is set in the keys variable									   */
/***********************************************************************************/
unsigned char pcc_getbutton	(void)
{	
	unsigned char keys;

  	cli();                                  /* read and clear atomic */
  	keys = pcc_data.keys.press;             /* read key(s)			 */
  	pcc_data.keys.press =  0;               /* clear key(s)			 */
  	sei();

	return keys;
}

/***********************************************************************************/
/*       led services                                            				   */
/***********************************************************************************/

/***********************************************************************************/
/* the isr occurs on compare 													   */
/* it debounces keys and serves their repeat function    						   */
/***********************************************************************************/
ISR(TIMER1_COMPA_vect)			       
{
	static unsigned char ct0, ct1;
  	unsigned char i;

    if(pcc_data.control.pc_time[0]) pcc_data.control.pc_time[0]--;
    if(pcc_data.control.pc_time[1]) pcc_data.control.pc_time[1]--;
	
	/* debounce keys */
	i = pcc_data.keys.state ^ ~PIND;                    // key changed ?
	ct0 = ~( ct0 & i );                             	// reset or count ct0
	ct1 = ct0 ^ (ct1 & i);                          	// reset or count ct1
	i &= ct0 & ct1;                                 	// count until roll over ?
	pcc_data.keys.state ^= i;                           // then toggle debounced state
	pcc_data.keys.press |= pcc_data.keys.state & i;     // 0->1: key press detect
 
	/* possibly serve repeat keys */
	#if 0
	if( (key_state & PCC_REPEAT_MASK) == 0 )            // check repeat function
    {
    	rpt = PCC_REPEAT_START;                         // start delay
	}
	if( --rpt == 0 )
	{
    	rpt = PCC_REPEAT_NEXT;                          // repeat delay
    	key_rpt |= key_state & PCC_REPEAT_MASK;
  	}
	#endif
}


