
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include "pcc_int.h"

/***********************************************************************************/
/***********************************************************************************/
void	pcc_control	(void)
{
	unsigned char i, button;

	/* we start up with pc outlet switched on -> if pc powers up directly after dc switch on */
	pcc_data.control.pc_time[0] = 
	pcc_data.control.pc_time[1] = PCC_TIME_WAIT_FOR_POWER_GOOD;
	
	pcc_data.control.pwr_good_and_relais = PCC_RELAIS_PC_BOTH;
	
	sei();			      		/* enable interrupts */

	for(;;) 
	{
		button=pcc_getbutton();

		for(i=0;i<2;i++)
		{
			if( button & (PCC_IN_PC<<i) )
			{
				/* soft power on button was pressed -> switch on the pc outlet */
				pcc_data.control.pwr_good_and_relais |= (PCC_RELAIS_PC<<i);
				
				cli();	/* two byte variable must be set while interrupt disabled */ 		      		
				pcc_data.control.pc_time[i] = PCC_TIME_WAIT_FOR_POWER_GOOD;
				sei();			      		

			}

			if( (PINA & (PCC_IN_PC<<i)) )
			{
				//pcc_data.control.pwr_good_and_relais |= (PCC_RELAIS_PC<<i);
			}
			else
			{
				if( pcc_data.control.pc_time[i] == 0 )
				{
					/* no "power good" was detected -> switch off the pc outlet */
					pcc_data.control.pwr_good_and_relais &= ~(PCC_RELAIS_PC<<i);
				}
			}
		}

		if /* at least one pc is powered */
		  (pcc_data.control.pwr_good_and_relais & PCC_RELAIS_PC_BOTH)
		{
			/* then switch on the monitor outlet */
			pcc_data.control.pwr_good_and_relais |= PCC_RELAIS_MONITOR;
		}
		else
		{
			/* otherwise every output can be set to 0 */
			pcc_data.control.pwr_good_and_relais = 0;
		}

		/* mirror the power good detection to the power good leds (green led and push button led) */ 
		pcc_data.control.pwr_good_and_relais &= ~0x03;
		pcc_data.control.pwr_good_and_relais |= (PINA & 0x03);
		
		PORTB = pcc_data.control.pwr_good_and_relais;
	}
}

/***********************************************************************************/
/* the main function initializes the hardware and the variables, then enters       */
/* the configmode and enters the gamemode after configuration is completed		   */
/***********************************************************************************/
int	main	(void)
{ 
	PORTA = 0x00;  			/* no input has no pullups 		*/
	DDRA  = 0x00;  			/* port A is input      		*/
	PORTB = 0x00;  			/* set all outputs to 0    		*/
	DDRB  = 0x1F;   		/* port B bit 0..4 are output	*/
	PORTD = 0x3F;  			/* port D bit 0..6 have pullups */
	DDRD  = 0x00;   		/* port D bit 0..6 are input	*/

	TCCR1A = 0b00000000;	
	TCCR1B = 0b00001100;	/* prescaler 256; CPU-Clock */
	TCCR1C = 0b00000000;	
	TIMSK  = 0b01000000;	/* interrupt at compare match */
	OCR1A  = 316;  	        /* timer compare interrupt after each 0,01s */
	

	pcc_key_init();       		/* initialize key variables */

	pcc_control();   			/* here we start controlling the pc */

	for(;;);
}
//***********************************************************************************//
